@@ -3,7 +3,7 @@ package Tree::Simple;
 use strict;
 use warnings;
 
-our $VERSION = '1.26';
+our $VERSION = '1.33';
 
 use Scalar::Util qw(blessed);
 
@@ -136,6 +136,11 @@ sub addChildren {
     goto &insertChildren;
 }
 
+sub generateChild
+{
+	return $_[0]->addChild($_[0]->new($_[1]) );
+}
+
 sub _insertChildAt {
     my ($self, $index, @trees) = @_;
 
@@ -346,6 +351,14 @@ sub getAllSiblings {
     $self->getParent()->getAllChildren();
 }
 
+sub getSiblingCount
+{
+	my($self) = @_;
+
+	return $self->isRoot ? 0 : $#{$self->getAllSiblings};
+
+} # End of getSiblingCount.
+
 ## -----------------------------------------------
 ## informational
 
@@ -365,6 +378,22 @@ sub size {
     return $size;
 }
 
+sub isFirstChild
+{
+	my($self) = @_;
+
+	return $self->isRoot ? 0 : $_[0]->getIndex == 0;
+
+} # End of isFirstChild.
+
+sub isLastChild
+{
+	my($self) = @_;
+
+	return $self->isRoot ? 0 : $self->getIndex == ($self->getParent->getChildCount - 1);
+
+} # End of isLastChild.
+
 ## -----------------------------------------------
 ## misc
 
@@ -440,8 +469,14 @@ sub traverse {
     (ref($post) eq "CODE") || die "Incorrect Object Type : post traversal function is not a function"
         if defined($post);
     foreach my $child ($self->getAllChildren()) {
-        $func->($child);
-        $child->traverse($func, $post);
+        my $ret = $func->($child) || '';
+
+        # Propagate up the stack.
+        return $ret if 'ABORT' eq $ret;
+
+        $ret = $child->traverse($func, $post) || '';
+        return $ret if 'ABORT' eq $ret;
+
         defined($post) && $post->($child);
     }
 }
@@ -602,11 +637,11 @@ Tree::Simple - A simple tree object
   # make a tree root
   my $tree = Tree::Simple->new("0", Tree::Simple->ROOT);
 
-  # explicity add a child to it
+  # explicitly add a child to it
   $tree->addChild(Tree::Simple->new("1"));
 
   # specify the parent when creating
-  # an instance and it adds the child implicity
+  # an instance and it adds the child implicitly
   my $sub_tree = Tree::Simple->new("2", $tree);
 
   # chain method calls
@@ -627,6 +662,23 @@ Tree::Simple - A simple tree object
   # clean up circular references
   $tree->DESTROY();
 
+Alternately, to avoid calling Tree::Simple->new(...) just to add a node:
+
+	use Tree::Simple;
+	use Data::TreeDumper; # Provides DumpTree().
+
+	# ---------------
+
+	my($root) = Tree::Simple->new('Root', Tree::Simple->ROOT);
+
+	$root->generateChild('Child 1.0');
+	$root->generateChild('Child 2.0');
+	$root->getChild(0)->generateChild('Grandchild 1.1');
+
+	print DumpTree($root);
+
+	$root->DESTROY;
+
 =head1 DESCRIPTION
 
 This module in an fully object-oriented implementation of a simple n-ary
@@ -727,6 +779,11 @@ Or the more complex:
                                      )
                          );
 
+=item B<generateChild ($scalar)>
+
+This method accepts a scalar and calls addChild(Tree::Simple->new($scalar) ) purely to
+save you the effort of needing to use C<< Tree::Simple->new(...) >> as the parameter.
+
 =item B<addChildren (@trees)>
 
 This method accepts an array of B<Tree::Simple> objects, and adds them to
@@ -826,6 +883,21 @@ It will return an array reference in scalar context.
 Much like C<addSibling> and C<addSiblings>, these two methods simply call
 C<getChild> and C<getAllChildren> on the parent of the invocant.
 
+See also </getSiblingCount>.
+
+Warning: This method includes the invocant, so it is not really all siblings but rather all
+children of the parent!
+
+=item B<getSiblingCount>
+
+Returns 0 if the invocant is the root node. Otherwise returns the count of siblings, which excludes
+the invocant.
+
+See also </getAllSiblings>.
+
+Warning: This differs from scalar(parent->getAllSiblings() ) just above, which for some reason
+includes the invocant. I cannot change getAllSiblings() now for a module first released in 2004.
+
 =item B<getDepth>
 
 Returns a number representing the depth of the invocant within the hierarchy of
@@ -875,6 +947,18 @@ Returns true (1) if the invocant does not have any children, false (0) otherwise
 Returns true (1) if the invocant has a "parent" of B<ROOT>, returns false
 (0) otherwise.
 
+=item B<isFistChild>
+
+Returns 0 if the invocant is the root node.
+
+Returns 1 if the invocant is the first child in the parental list of children. Otherwise returns 0.
+
+=item B<isLastChild>
+
+Returns 0 if the invocant is the root node.
+
+Returns 1 if the invocant is the last child in the parental list of children. Otherwise returns 0.
+
 =back
 
 =head2 Recursive Methods
@@ -887,16 +971,18 @@ This method accepts two arguments a mandatory C<$func> and an optional
 C<$postfunc>. If the argument C<$func> is not defined then an exception
 is thrown. If C<$func> or C<$postfunc> are not in fact CODE references
 then an exception is thrown. The function C<$func> is then applied
-recursively to all the children of the invocant. If given, the function
-C<$postfunc> will be applied to each child after the children of the child
-have been traversed.
+recursively to all the children of the invocant, or until C<$func>
+returns C<'ABORT'>. If given, the function C<$postfunc> will be applied
+to each child after the children of the child have been traversed.
 
 Here is an example of a traversal function that will print out the
 hierarchy as a tabbed in list.
 
   $tree->traverse(sub {
       my ($_tree) = @_;
-      print (("\t" x $_tree->getDepth()), $_tree->getNodeValue(), "\n");
+      my $tag = $_tree->getNodeValue();
+      print (("\t" x $_tree->getDepth()), $tag, "\n");
+      return 'ABORT' if 'foo' eq $tag;
   });
 
 Here is an example of a traversal function that will print out the
@@ -913,6 +999,10 @@ hierarchy in an XML-style format.
               '</', $_tree->getNodeValue(),'>',"\n");
   });
 
+Note that aborting traverse is not recommended when using C<$postfunc>
+because post-function will not be called for any nodes after aborting
+which might lead to less than predictable results.
+
 =item B<size>
 
 Returns the total number of nodes in the current tree and all its sub-trees.
@@ -1192,10 +1282,9 @@ specialized in purpose.
 =item L<Tree::DAG_Node>
 
 This module seems pretty stable and very robust with a lot of functionality.
-However, B<Tree::DAG_Node> does not come with any automated tests.
-I<test.pl> file simply checks the module loads and nothing else. While I
-am sure the author tested his code, I would feel better if I was able to
-see that. The module is approx. 3000 lines with POD, and 1,500 without the
+But it> only comes with 1 sophisticated test, t/cut.and.paste.subtrees.t.
+While I am sure the author tested his code, I would feel better if I was able
+to see that. The module is approx. 3000 lines with POD, and 1,500 without the
 POD. The shear depth and detail of the documentation and the ratio of code
 to documentation is impressive, and not to be taken lightly. But given that
 it is a well known fact that the likeliness of bugs increases along side the
